/*
     Some examples of pointers and arrays.

     If you need help deciphering the pointer declarations,
     remember this strategy: Start with the variable name,
     work right until you get to a right parentheses, then go
     back to the variable and work left until you get to a
     left parenthesis, then go out a level of parentheses and repeat.

     This program also demonstrates how memory can be allocated
     for variables in three different storage areas:
        statically    - in the data segment,
        automatically - on the stack,
        dynamically   - in the heap.
*/
#include <stdio.h>
#include <stdlib.h>


int (a[3])[4];    // 48 bytes statically allocated in the data segment

int (*b[3])[4];   // 12 bytes statically allocated in the data segment

int *c[3][4];      // 4 bytes statically allocated in the data segment

int (*d)[3][4];    // 4 bytes statically allocated in the data segment

int (*(*e)[3])[4]; // 4 bytes statically allocated in the data segment


int main()
{
   int i, j;  // loop variables

   // dynamically allocate (in the heap) rows for the array b
   b[0] = calloc(4, sizeof(int));
   b[1] = calloc(4, sizeof(int));
   //b[2] = calloc(4, sizeof(int));

   // automatically allocate (on the stack) rows for the array b
   //int b0[4];
   //int b1[4];
   int b2[4];

   //b[0] = &b0;
   //b[1] = &b1;
   b[2] = &b2;

   // Notice that the above code has allocated two rows of b[]
   // in the heap and one row in the stack. This is a bit silly,
   // but it does demonstrate very important ideas. You can, of
   // course, change this to put the whole array in the heap or
   // on the stack.

   // initialize the arrays a and b
   for (i = 0; i < 3; i++)
     for (j = 0; j < 4; j++)
     {
        a[i][j] = i*j;
        (*b[i])[j] = i*j;
//      b[i][j] = i*j;  // doesn't work (try it)
     }

   // The next two lines show that d is a pointer to the type of a
   // and e is a pointer to the type of b.
   d = &a;
   e = &b;

   d = a;  // generates a warning
   e = b;  // generates a warning


   // Here is a nice trick. Suppose you are having trouble reading a
   // pointer declaration (like d and e). Have the LCC compiler read it
   // for you. If you force an error or warning from the LCC compiler, it
   // generates very readable messages. Try this with the next line. LCC
   // will "tell you" what kind of pointer each of d and e is.
   // (The gcc and Visual Studio compilers do not give messages that are
   //  any where near as helpful as LCC's.)
   //d = e;


   // print the arrays
   for (i = 0; i < 3; i++)
   { for (j = 0; j < 4; j++)
        printf("%d ", a[i][j]);
     printf("\n");
   }

   printf("\n");

   for (i = 0; i < 3; i++)
   { for (j = 0; j < 4; j++)
        printf("%d ", (*b[i])[j]);
     printf("\n");
   }

   return 0;
}
